%{
#include <stdio.h>
#include "ghparser.h"
extern int yylex(void);
extern void yyerror(char const *);
#define YYERROR_VERBOSE 1
#define YY_USER_INIT yylloc.first_line=1;
%}

%start game

%union {
    const char* str;
    int	num;
    pointer ptr;
}

%error-verbose

%token DOTGAME;
%token DOTMAP DOTM DOTR DOTC DOTD DOTT COMMA;
%token SLASH SEMI RBRACKET LBRACKET BANG;

%token<str> PCTDOL PCTSLASH PCTT PCTAT PLUS MINUS;
%token<str> DOTO DOTX DOTF DOTFQ DOTFN DOTXI DOTXO DOTU DOTV;
%token<str> WORD FILEPART STRING QUOTE ATFLAG;
%token<num> INT;

%type <num> radius opt-bang plus-or-minus;
%type <str> image opt-image fname label animation desc TEXT;
%type <str> thingname normfname arrowname icon;
%type <str> ID option-letter;
%type <ptr> money-condition moves-condition have-condition flag-condition;
%type <ptr> condition-main cond-text vis-condition use-condition;

%%
TEXT: STRING { $$ = $1; }
game: gamehead rooms map { outputResults(); }
gamehead: DOTGAME WORD { setGameName($2); };
map: DOTMAP fname { SetMap($2); };

setflags: %empty
	|
	setflags setflag;
setflag: ATFLAG ID { setFlag($1, $2); }

rooms:	room
	|
	rooms room;
mapdot: %empty { }
	|
	mapdot-head opt-cond { PopChoices(); };

mapdot-head: DOTM label INT COMMA INT radius
	    { AddMapDot($2, $3, $5, $6); };
label:  %empty { $$ = ""; }
	|
	WORD COMMA;
radius:  %empty { $$ = 0; }
	|
	COMMA INT { $$ = $2; };
room: room-header room-insides { };
room-names: DOTR ID WORD 
	{ AddRoom($3, $2); };
room-header: room-names opt-imgs mapdot;
opt-imgs:  %empty { }
	|
	COMMA image COMMA animation { SetImages($2, $4); }
	|
	COMMA image { SetImages($2, ""); } ;
image: fname { $$ = $1; };
opt-image: %empty { $$ = ""; }
	|
	image {$$ = $1; };
animation: fname { $$ = $1; };
room-insides: LBRACKET room-contents description RBRACKET { PopChoices(); }
room-contents: %empty
	|
	room-contents character
	|
	room-contents door
	|
	room-contents thing;
desc: %empty { $$ = ""; }
	|
	desc STRING { $$ = mystrcat(mystrcat($1, "\n"), $2); }
description: desc { setDesc($1); };
//                              room-id
//                                 v
door: door-head opt-cond { PopChoices(); };
door-head: DOTD ID arrowname SLASH ID { AddDoor($2, $3, $5); };
arrowname: WORD;
opt-cond: %empty 
	|
	condition;
character: char-head character-insides { PopChoices(); };
character-insides: LBRACKET options description RBRACKET;
char-head: char-names opt-imgs opt-cond;
char-names: DOTC ID WORD { AddCharacter($3, $2); };
options: %empty
	|
	options option;
option-contents: %empty
	|
	LBRACKET setflags options RBRACKET { PopChoices(); };
option: option-head opt-cond option-contents;
option-head: option-letter ID TEXT { AddOption($1, $2, $3); };
option-letter: DOTO | DOTX | DOTF;
thing: thing-head opt-cond description { PopChoices(); } ;
thing-head: DOTT ID thingname image icon 
		{ AddThing($2, $3, $4, $5); };
thingname: WORD;
icon: SLASH fname { $$ = $2; };
vis-condition: DOTV condition-main opt-image
		{ AddCondition('v', $2, $3); };
use-condition: DOTU condition-main cond-text
		{ AddCondition('u', $2, $3); } ;
condition: vis-condition
	|
	use-condition;
cond-text: %empty  { $$ = ""; }
	|
	STRING { $$ = (char*)$1; };
condition-main: money-condition { $$ = $1; }
	|
	have-condition { $$ = $1; }
	|
	moves-condition { $$ = $1; }
	|
	flag-condition { $$ = $1; } ;
money-condition: PCTDOL plus-or-minus INT { MkCondInt($1, $2, $3);} ;
have-condition: PCTSLASH opt-bang ID { MkCondID($1, $2, $3); };
moves-condition: PCTT plus-or-minus INT { MkCondInt($1, $2, $3); } ;
flag-condition: PCTAT opt-bang ID { MkCondID($1, $2, $3); };
plus-or-minus: PLUS { $$ = 1; }
	|
	MINUS { $$ = -1; };
opt-bang: %empty { $$ = TRUE; }
	|
	BANG { $$ = FALSE; };
normfname: FILEPART
	|
	SLASH FILEPART { $$ = CombineFName("", $2); }
	|
	normfname SLASH FILEPART { $$ = CombineFName($1, $3); }
	|
	fname SLASH { $$ = CombineFName($1, ""); }
fname:	normfname
	|
	QUOTE;
ID: WORD;
%%
